@@ -12,7 +12,8 @@
  *
  *     https://opensource.org/licenses/BSD-3-Clause
  */
-#define _QNX_SOURCE /* sleep */
+
+#define _QNX_SOURCE /* sleep() */
 #define _GNU_SOURCE
 
 #include <inttypes.h>
@@ -143,10 +144,13 @@ print_current_config(sr_session_ctx_t *session, const char *module_name)
     sr_val_t *values = NULL;
     size_t count = 0;
     int rc = SR_ERR_OK;
-    char xpath[128];
+    char *xpath;
 
-    sprintf(xpath, "/%s:*//.", module_name);
+    if (asprintf(&xpath, "/%s:*//.", module_name) == -1) {
+        return SR_ERR_NO_MEMORY;
+    }
     rc = sr_get_items(session, xpath, 0, 0, &values, &count);
+    free(xpath);
     if (rc != SR_ERR_OK) {
         return rc;
     }
@@ -174,7 +178,7 @@ ev_to_str(sr_event_t ev)
 }
 
 static int
-module_change_cb(sr_session_ctx_t *session, const char *module_name, const char *xpath, sr_event_t event,
+module_change_cb(sr_session_ctx_t *session, uint32_t sub_id, const char *module_name, const char *xpath, sr_event_t event,
         uint32_t request_id, void *private_data)
 {
     sr_change_iter_t *it = NULL;
@@ -184,6 +188,7 @@ module_change_cb(sr_session_ctx_t *session, const char *module_name, const char
     sr_val_t *old_value = NULL;
     sr_val_t *new_value = NULL;
 
+    (void)sub_id;
     (void)request_id;
     (void)private_data;
 
