@@ -4,25 +4,17 @@
  * @brief sysrepo plugin daemon
  *
  * @copyright
- * Copyright 2018 Deutsche Telekom AG.
- * Copyright 2018 - 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
-#define _POSIX_C_SOURCE 200809L /* sigaction */
-#define _DEFAULT_SOURCE /* struct dirent.d_type */
-#define _GNU_SOURCE /* asprintf */
-#define _XOPEN_SOURCE 500 /* strdup */
+
+#define _GNU_SOURCE
 
 #include <dirent.h>
 #include <dlfcn.h>
@@ -222,6 +214,7 @@ sr_mkpath(const char *path, mode_t mode)
         if (mkdir(dup, mode) == -1) {
             if (errno != EEXIST) {
                 *p = '/';
+                free(dup);
                 return -1;
             }
         }
@@ -265,7 +258,7 @@ load_plugins(struct srpd_plugin_s **plugins, int *plugin_count)
     /* get plugins dir from environment variable, or use default one */
     plugins_dir = getenv("SRPD_PLUGINS_PATH");
     if (!plugins_dir) {
-        plugins_dir = SRPD_PLUGINS_PATH;
+        plugins_dir = SRPD_PLG_PATH;
     }
 
     /* create the directory if it does not exist */
