@@ -4,24 +4,17 @@
  * @brief sysrepocfg tool
  *
  * @copyright
- * Copyright 2018 Deutsche Telekom AG.
- * Copyright 2018 - 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
-#define _GNU_SOURCE /* asprintf */
-#define _DEFAULT_SOURCE /* mkstemps */
-#define _XOPEN_SOURCE 500 /* mkstemp */
+
+#define _GNU_SOURCE
 
 #include <errno.h>
 #include <getopt.h>
@@ -41,6 +34,8 @@
 #include "compat.h"
 #include "sysrepo.h"
 
+sr_log_level_t log_level = SR_LL_ERR;
+
 static void
 version_print(void)
 {
@@ -55,45 +50,52 @@ help_print(void)
 {
     printf(
             "Usage:\n"
-            "  sysrepocfg <operation-option> [other-options]\n"
+            "  sysrepocfg <operation> [options]\n"
             "\n"
-            "Available operation-options:\n"
-            "  -h, --help                   Prints usage help.\n"
-            "  -V, --version                Prints only information about sysrepo version.\n"
-            "  -I, --import[=<file-path>]   Import the configuration from a file or STDIN.\n"
-            "  -X, --export[=<file-path>]   Export configuration to a file or STDOUT.\n"
-            "  -E, --edit[=<file-path>/<editor>]\n"
+            "Available operations:\n"
+            "  -h, --help                   Print usage help.\n"
+            "  -V, --version                Print only information about sysrepo version.\n"
+            "  -I, --import[=<path>]        Import the configuration from a file or STDIN.\n"
+            "  -X, --export[=<path>]        Export configuration to a file or STDOUT.\n"
+            "  -E, --edit[=<path>/<editor>]\n"
             "                               Edit configuration data by merging (applying) a configuration (edit) file or\n"
             "                               by editing the current datastore content using a text editor.\n"
-            "  -R, --rpc[=<file-path>/<editor>]\n"
+            "  -R, --rpc[=<path>/<editor>]\n"
             "                               Send a RPC/action in a file or using a text editor. Output is printed to STDOUT.\n"
-            "  -N, --notification[=<file-path>/<editor>]\n"
+            "  -N, --notification[=<path>/<editor>]\n"
             "                               Send a notification in a file or using a text editor.\n"
-            "  -C, --copy-from <file-path>/<source-datastore>\n"
+            "  -C, --copy-from <path>/<source-datastore>\n"
             "                               Perform a copy-config from a file or a datastore.\n"
-            "  -W, --new-data <file-path>   Set the configuration from a file as the initial one for a new module only scheduled\n"
+            "  -W, --new-data <path>        Set the configuration from a file as the initial one for a new module only scheduled\n"
             "                               to be installed. Is useful for modules with mandatory top-level nodes.\n"
             "\n"
-            "       When both a <file-path> and <editor>/<target-datastore> can be specified, it is always first checked\n"
+            "       When both a <path> and <editor>/<source-datastore> can be specified, it is always first checked\n"
             "       that the file exists. If not, then it is interpreted as the other parameter.\n"
-            "       If no <file-path> and no <editor> is set, use text editor in $VISUAL or $EDITOR environment variables.\n"
+            "       If no <path> and no <editor> is set, use text editor in $VISUAL or $EDITOR environment variables.\n"
             "\n"
-            "Available other-options:\n"
+            "Available options:\n"
             "  -d, --datastore <datastore>  Datastore to be operated on, \"running\" by default (\"running\", \"startup\",\n"
-            "                               \"candidate\", or \"operational\") (import, export, edit, copy-from op).\n"
-            "  -m, --module <module-name>   Module to be operated on, otherwise it is operated on full datastore\n"
-            "                               (import, export, edit, copy-from, mandatory for new-data op).\n"
-            "  -x, --xpath <xpath>          XPath to select (export op).\n"
+            "                               \"candidate\", or \"operational\"). Accepted by import, export, edit,\n"
+            "                               copy-from op.\n"
+            "  -m, --module <module-name>   Module to be operated on, otherwise it is operated on full datastore.\n"
+            "                               Accepted by import, export, edit, copy-from, mandatory for new-data op.\n"
+            "  -x, --xpath <xpath>          XPath to select. Accepted by export op.\n"
             "  -f, --format <format>        Data format to be used, by default based on file extension or \"xml\" if not applicable\n"
-            "                               (\"xml\", \"json\", or \"lyb\") (import, export, edit, rpc, notification, copy-from, new-data op).\n"
-            "  -l, --lock                   Lock the specified datastore for the whole operation (edit op).\n"
-            "  -n, --not-strict             Silently ignore any unknown data (import, edit, rpc, notification, copy-from op).\n"
-            "  -p, --depth <number>         Limit the depth of returned subtrees, 0 so unlimited by default (export op).\n"
+            "                               (\"xml\", \"json\", or \"lyb\"). Accepted by import, export, edit, rpc,\n"
+            "                               notification, copy-from, new-data op.\n"
+            "  -l, --lock                   Lock the specified datastore for the whole operation. Accepted by edit op.\n"
+            "  -n, --not-strict             Silently ignore any unknown data. Accepted by import, edit, rpc, notification,\n"
+            "                               copy-from op.\n"
+            "  -o, --opaque                 Parse invalid nodes in the edit into opaque nodes. Accepted by edit op.\n"
+            "  -p, --depth <depth>          Limit the depth of returned subtrees, 0 (unlimited) by default. Accepted by\n"
+            "                               export op.\n"
             "  -t, --timeout <seconds>      Set the timeout for the operation, otherwise the default one is used.\n"
-            "  -w, --wait                   Wait for all the callbacks to be called on a data change including DONE or ABORT.\n"
-            "  -e, --defaults <wd-mode>     Print the default values, which are hidden by default (\"report-all\",\n"
-            "                               \"report-all-tagged\", \"trim\", \"explicit\", \"implicit-tagged\") (export, edit, rpc op).\n"
-            "  -v, --verbosity <level>      Change verbosity to a level (none, error, warning, info, debug) or number (0, 1, 2, 3, 4).\n"
+            "                               Accepted by all op.\n"
+            "  -e, --defaults <wd-mode>     Print the default values, which are trimmed by default (\"report-all\",\n"
+            "                               \"report-all-tagged\", \"trim\", \"explicit\", \"implicit-tagged\").\n"
+            "                               Accepted by export, edit, rpc op.\n"
+            "  -v, --verbosity <level>      Change verbosity to a level (none, error, warning, info, debug) or\n"
+            "                               number (0, 1, 2, 3, 4).\n"
             "\n");
 }
 
@@ -112,18 +114,22 @@ error_print(int sr_error, const char *format, ...)
     va_start(ap, format);
     vfprintf(stderr, msg, ap);
     va_end(ap);
+
+    if (log_level < SR_LL_INF) {
+        fprintf(stderr, "For more details you may try to increase the verbosity up to \"-v3\".\n");
+    }
 }
 
 static void
-error_ly_print(struct ly_ctx *ctx)
+error_ly_print(const struct ly_ctx *ctx)
 {
     struct ly_err_item *e;
 
     for (e = ly_err_first(ctx); e; e = e->next) {
-        error_print(0, "libyang: %s", e->msg);
+        fprintf(stderr, "libyang error: %s\n", e->msg);
     }
 
-    ly_err_clean(ctx, NULL);
+    ly_err_clean((struct ly_ctx *)ctx, NULL);
 }
 
 static int
@@ -202,13 +208,24 @@ step_read_file(FILE *file, char **mem)
     return EXIT_SUCCESS;
 }
 
-static int
-step_load_data(sr_session_ctx_t *sess, const char *file_path, LYD_FORMAT format, int flags, struct lyd_node **data)
-{
-    struct ly_ctx *ly_ctx;
-    char *ptr;
+enum data_type {
+    DATA_CONFIG,
+    DATA_EDIT,
+    DATA_RPC,
+    DATA_NOTIF
+};
 
-    ly_ctx = (struct ly_ctx *)sr_get_context(sr_session_get_connection(sess));
+static int
+step_load_data(sr_session_ctx_t *sess, const char *file_path, LYD_FORMAT format, enum data_type data_type,
+        int not_strict, int opaq, struct lyd_node **data)
+{
+    const struct ly_ctx *ly_ctx;
+    struct ly_in *in;
+    char *ptr;
+    int parse_flags;
+    LY_ERR lyrc = 0;
+
+    ly_ctx = sr_get_context(sr_session_get_connection(sess));
 
     /* learn format */
     if (format == LYD_UNKNOWN) {
@@ -230,18 +247,41 @@ step_load_data(sr_session_ctx_t *sess, const char *file_path, LYD_FORMAT format,
         }
     }
 
-    /* parse import data */
+    /* get input */
     if (file_path) {
-        *data = lyd_parse_path(ly_ctx, file_path, format, flags, NULL);
+        if (ly_in_new_filepath(file_path, 0, &in)) {
+            /* empty file */
+            ptr = strdup("");
+            ly_in_new_memory(ptr, &in);
+        }
     } else {
         /* we need to load the data into memory first */
         if (step_read_file(stdin, &ptr)) {
             return EXIT_FAILURE;
         }
-        *data = lyd_parse_mem(ly_ctx, ptr, format, flags);
-        free(ptr);
+        ly_in_new_memory(ptr, &in);
     }
-    if (ly_errno) {
+
+    /* parse data */
+    switch (data_type) {
+    case DATA_CONFIG:
+        parse_flags = LYD_PARSE_NO_STATE | LYD_PARSE_ONLY | (not_strict ? 0 : LYD_PARSE_STRICT);
+        lyrc = lyd_parse_data(ly_ctx, NULL, in, format, parse_flags, 0, data);
+        break;
+    case DATA_EDIT:
+        parse_flags = LYD_PARSE_NO_STATE | LYD_PARSE_ONLY | (opaq ? LYD_PARSE_OPAQ : (not_strict ? 0 : LYD_PARSE_STRICT));
+        lyrc = lyd_parse_data(ly_ctx, NULL, in, format, parse_flags, 0, data);
+        break;
+    case DATA_RPC:
+        lyrc = lyd_parse_op(ly_ctx, NULL, in, format, LYD_TYPE_RPC_YANG, data, NULL);
+        break;
+    case DATA_NOTIF:
+        lyrc = lyd_parse_op(ly_ctx, NULL, in, format, LYD_TYPE_NOTIF_YANG, data, NULL);
+        break;
+    }
+    ly_in_free(in, 1);
+
+    if (lyrc) {
         error_ly_print(ly_ctx);
         error_print(0, "Data parsing failed");
         return EXIT_FAILURE;
@@ -293,15 +333,14 @@ op_import(sr_session_ctx_t *sess, const char *file_path, const char *module_name
         int timeout_s)
 {
     struct lyd_node *data;
-    int r, flags;
+    int r;
 
-    flags = LYD_OPT_CONFIG | LYD_OPT_TRUSTED | (not_strict ? 0 : LYD_OPT_STRICT);
-    if (step_load_data(sess, file_path, format, flags, &data)) {
+    if (step_load_data(sess, file_path, format, DATA_CONFIG, not_strict, 0, &data)) {
         return EXIT_FAILURE;
     }
 
     /* replace config (always spends data) */
-    r = sr_replace_config(sess, module_name, data, timeout_s * 1000, 1);
+    r = sr_replace_config(sess, module_name, data, timeout_s * 1000);
     if (r) {
         error_print(r, "Replace config failed");
         return EXIT_FAILURE;
@@ -333,9 +372,12 @@ op_export(sr_session_ctx_t *sess, const char *file_path, const char *module_name
 
     /* get subtrees */
     if (module_name) {
-        asprintf(&str, "/%s:*", module_name);
-        r = sr_get_data(sess, str, max_depth, timeout_s * 1000, 0, &data);
-        free(str);
+        if (asprintf(&str, "/%s:*", module_name) == -1) {
+            r = SR_ERR_NO_MEMORY;
+        } else {
+            r = sr_get_data(sess, str, max_depth, timeout_s * 1000, 0, &data);
+            free(str);
+        }
     } else if (xpath) {
         r = sr_get_data(sess, xpath, max_depth, timeout_s * 1000, 0, &data);
     } else {
@@ -350,8 +392,8 @@ op_export(sr_session_ctx_t *sess, const char *file_path, const char *module_name
     }
 
     /* print exported data */
-    lyd_print_file(file ? file : stdout, data, format, LYP_FORMAT | LYP_WITHSIBLINGS | wd_opt);
-    lyd_free_withsiblings(data);
+    lyd_print_file(file ? file : stdout, data, format, LYD_PRINT_WITHSIBLINGS | wd_opt);
+    lyd_free_all(data);
 
     /* cleanup */
     if (file) {
@@ -362,27 +404,31 @@ op_export(sr_session_ctx_t *sess, const char *file_path, const char *module_name
 
 static int
 op_edit(sr_session_ctx_t *sess, const char *file_path, const char *editor, const char *module_name, LYD_FORMAT format,
-        int lock, int not_strict, int wd_opt, int timeout_s)
+        int lock, int not_strict, int opaq, int wd_opt, int timeout_s)
 {
     char tmp_file[22];
-    int r, flags, rc = EXIT_FAILURE;
+    int r, rc = EXIT_FAILURE;
     struct lyd_node *data;
 
     if (file_path) {
         /* just apply an edit from a file */
-        flags = LYD_OPT_EDIT | LYD_OPT_TRUSTED | (not_strict ? 0 : LYD_OPT_STRICT);
-        if (step_load_data(sess, file_path, format, flags, &data)) {
+        if (step_load_data(sess, file_path, format, DATA_EDIT, not_strict, opaq, &data)) {
+            return EXIT_FAILURE;
+        }
+
+        if (!data) {
+            error_print(0, "No parsed data");
             return EXIT_FAILURE;
         }
 
         r = sr_edit_batch(sess, data, "merge");
-        lyd_free_withsiblings(data);
+        lyd_free_all(data);
         if (r != SR_ERR_OK) {
             error_print(r, "Failed to prepare edit");
             return EXIT_FAILURE;
         }
 
-        r = sr_apply_changes(sess, timeout_s * 1000, 1);
+        r = sr_apply_changes(sess, timeout_s * 1000);
         if (r != SR_ERR_OK) {
             error_print(r, "Failed to merge edit data");
             return EXIT_FAILURE;
@@ -428,11 +474,10 @@ cleanup_unlock:
 }
 
 static int
-op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FORMAT format, int not_strict, int wd_opt,
-        int timeout_s)
+op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FORMAT format, int wd_opt, int timeout_s)
 {
     char tmp_file[22];
-    int r, flags;
+    int r;
     struct lyd_node *input, *output, *node;
 
     if (!file_path) {
@@ -450,38 +495,37 @@ op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FO
     }
 
     /* load the file */
-    flags = LYD_OPT_RPC | LYD_OPT_TRUSTED | (not_strict ? 0 : LYD_OPT_STRICT);
-    if (step_load_data(sess, file_path, format, flags, &input)) {
+    if (step_load_data(sess, file_path, format, DATA_RPC, 0, 0, &input)) {
         return EXIT_FAILURE;
     }
 
     /* send rpc/action */
     r = sr_rpc_send_tree(sess, input, timeout_s * 1000, &output);
-    lyd_free_withsiblings(input);
+    lyd_free_all(input);
     if (r) {
         error_print(r, "Sending RPC/action failed");
         return EXIT_FAILURE;
     }
 
     /* print output if any */
-    LY_TREE_FOR(output->child, node) {
-        if (!node->dflt) {
+    LY_LIST_FOR(lyd_child(output), node) {
+        if (!(node->flags & LYD_DEFAULT)) {
             break;
         }
     }
     if (node) {
-        lyd_print_file(stdout, output, format, LYP_FORMAT | wd_opt);
+        lyd_print_file(stdout, output, format, wd_opt);
     }
-    lyd_free_withsiblings(output);
+    lyd_free_all(output);
 
     return EXIT_SUCCESS;
 }
 
 static int
-op_notif(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FORMAT format, int not_strict)
+op_notif(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FORMAT format)
 {
     char tmp_file[22];
-    int r, flags;
+    int r;
     struct lyd_node *notif;
 
     if (!file_path) {
@@ -499,14 +543,13 @@ op_notif(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_
     }
 
     /* load the file */
-    flags = LYD_OPT_NOTIF | LYD_OPT_TRUSTED | (not_strict ? 0 : LYD_OPT_STRICT);
-    if (step_load_data(sess, file_path, format, flags, &notif)) {
+    if (step_load_data(sess, file_path, format, DATA_NOTIF, 0, 0, &notif)) {
         return EXIT_FAILURE;
     }
 
     /* send notification */
-    r = sr_event_notif_send_tree(sess, notif);
-    lyd_free_withsiblings(notif);
+    r = sr_event_notif_send_tree(sess, notif, 0, 1);
+    lyd_free_all(notif);
     if (r) {
         error_print(r, "Sending notification failed");
         return EXIT_FAILURE;
@@ -519,25 +562,24 @@ static int
 op_copy(sr_session_ctx_t *sess, const char *file_path, sr_datastore_t source_ds, const char *module_name,
         LYD_FORMAT format, int not_strict, int timeout_s)
 {
-    int r, flags;
+    int r;
     struct lyd_node *data;
 
     if (file_path) {
         /* load the file */
-        flags = LYD_OPT_CONFIG | LYD_OPT_TRUSTED | (not_strict ? 0 : LYD_OPT_STRICT);
-        if (step_load_data(sess, file_path, format, flags, &data)) {
+        if (step_load_data(sess, file_path, format, DATA_CONFIG, not_strict, 0, &data)) {
             return EXIT_FAILURE;
         }
 
         /* replace data */
-        r = sr_replace_config(sess, module_name, data, timeout_s * 1000, 1);
+        r = sr_replace_config(sess, module_name, data, timeout_s * 1000);
         if (r) {
             error_print(r, "Replace config failed");
             return EXIT_FAILURE;
         }
     } else {
         /* copy config */
-        r = sr_copy_config(sess, module_name, source_ds, timeout_s * 1000, 1);
+        r = sr_copy_config(sess, module_name, source_ds, timeout_s * 1000);
         if (r) {
             error_print(r, "Copy config failed");
             return EXIT_FAILURE;
@@ -596,8 +638,7 @@ main(int argc, char **argv)
     LYD_FORMAT format = LYD_UNKNOWN;
     const char *module_name = NULL, *editor = NULL, *file_path = NULL, *xpath = NULL, *op_str;
     char *ptr;
-    sr_log_level_t log_level = SR_LL_ERR;
-    int r, rc = EXIT_FAILURE, opt, operation = 0, lock = 0, not_strict = 0, timeout = 0, wd_opt = 0;
+    int r, rc = EXIT_FAILURE, opt, operation = 0, lock = 0, not_strict = 0, opaq = 0, timeout = 0, wd_opt = 0;
     uint32_t max_depth = 0;
     struct option options[] = {
         {"help",            no_argument,       NULL, 'h'},
@@ -615,6 +656,7 @@ main(int argc, char **argv)
         {"format",          required_argument, NULL, 'f'},
         {"lock",            no_argument,       NULL, 'l'},
         {"not-strict",      no_argument,       NULL, 'n'},
+        {"opaque",          no_argument,       NULL, 'o'},
         {"depth",           required_argument, NULL, 'p'},
         {"timeout",         required_argument, NULL, 't'},
         {"defaults",        required_argument, NULL, 'e'},
@@ -629,7 +671,7 @@ main(int argc, char **argv)
 
     /* process options */
     opterr = 0;
-    while ((opt = getopt_long(argc, argv, "hVI::X::E::R::N::C:W:d:m:x:f:lnp:t:e:v:", options, NULL)) != -1) {
+    while ((opt = getopt_long(argc, argv, "hVI::X::E::R::N::C:W:d:m:x:f:lnop:t:e:v:", options, NULL)) != -1) {
         switch (opt) {
         case 'h':
             version_print();
@@ -767,6 +809,9 @@ main(int argc, char **argv)
         case 'n':
             not_strict = 1;
             break;
+        case 'o':
+            opaq = 1;
+            break;
         case 'p':
             max_depth = strtoul(optarg, &ptr, 10);
             if (ptr[0]) {
@@ -783,15 +828,15 @@ main(int argc, char **argv)
             break;
         case 'e':
             if (!strcmp(optarg, "report-all")) {
-                wd_opt = LYP_WD_ALL;
+                wd_opt = LYD_PRINT_WD_ALL;
             } else if (!strcmp(optarg, "report-all-tagged")) {
-                wd_opt = LYP_WD_ALL_TAG;
+                wd_opt = LYD_PRINT_WD_ALL_TAG;
             } else if (!strcmp(optarg, "trim")) {
-                wd_opt = LYP_WD_TRIM;
+                wd_opt = LYD_PRINT_WD_TRIM;
             } else if (!strcmp(optarg, "explicit")) {
-                wd_opt = LYP_WD_EXPLICIT;
+                wd_opt = LYD_PRINT_WD_EXPLICIT;
             } else if (!strcmp(optarg, "implicit-tagged")) {
-                wd_opt = LYP_WD_IMPL_TAG;
+                wd_opt = LYD_PRINT_WD_IMPL_TAG;
             } else {
                 error_print(0, "Invalid defaults mode \"%s\"", optarg);
                 goto cleanup;
@@ -874,13 +919,13 @@ main(int argc, char **argv)
         rc = op_export(sess, file_path, module_name, xpath, format, max_depth, wd_opt, timeout);
         break;
     case 'E':
-        rc = op_edit(sess, file_path, editor, module_name, format, lock, not_strict, wd_opt, timeout);
+        rc = op_edit(sess, file_path, editor, module_name, format, lock, not_strict, opaq, wd_opt, timeout);
         break;
     case 'R':
-        rc = op_rpc(sess, file_path, editor, format, not_strict, wd_opt, timeout);
+        rc = op_rpc(sess, file_path, editor, format, wd_opt, timeout);
         break;
     case 'N':
-        rc = op_notif(sess, file_path, editor, format, not_strict);
+        rc = op_notif(sess, file_path, editor, format);
         break;
     case 'C':
         rc = op_copy(sess, file_path, source_ds, module_name, format, not_strict, timeout);
