@@ -4,38 +4,32 @@
  * @brief sysrepoctl tool
  *
  * @copyright
- * Copyright 2018 Deutsche Telekom AG.
- * Copyright 2018 - 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE
-#define _XOPEN_SOURCE 500 /* strdup */
 
 #include <assert.h>
-#include <sys/stat.h>
+#include <getopt.h>
+#include <stdarg.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdint.h>
-#include <stdarg.h>
-#include <getopt.h>
+#include <sys/stat.h>
 
 #include <libyang/libyang.h>
 
+#include "bin_common.h"
 #include "compat.h"
 #include "sysrepo.h"
-#include "bin_common.h"
 
 #define SRCTL_LIST_NAME "Module Name"
 #define SRCTL_LIST_REVISION "Revision"
@@ -58,56 +52,63 @@ struct list_item {
     char *features;
 };
 
+sr_log_level_t log_level = SR_LL_ERR;
+
 static void
 version_print(void)
 {
     printf(
-        "sysrepoctl - sysrepo YANG schema manipulation tool, compiled with libsysrepo v%s (SO v%s)\n"
-        "\n",
-        SR_VERSION, SR_SOVERSION
-    );
+            "sysrepoctl - sysrepo YANG schema manipulation tool, compiled with libsysrepo v%s (SO v%s)\n"
+            "\n",
+            SR_VERSION, SR_SOVERSION);
 }
 
 static void
 help_print(void)
 {
     printf(
-        "Usage:\n"
-        "  sysrepoctl <operation-option> [other-options]\n"
-        "\n"
-        "Available operation-options:\n"
-        "  -h, --help           Print usage help.\n"
-        "  -V, --version        Print only information about sysrepo version.\n"
-        "  -l, --list           List YANG modules in sysrepo.\n"
-        "  -i, --install <path> Install the specified schema into sysrepo. Can be in either YANG or YIN format.\n"
-        "  -u, --uninstall <module>[,<module2>,<module3> ...]\n"
-        "                       Uninstall the specified module(s) from sysrepo.\n"
-        "  -c, --change <module>\n"
-        "                       Change access rights, features, or replay support of the specified module.\n"
-        "                       Use special \":ALL\" module name to change the replay support of all the modules.\n"
-        "  -U, --update <path>  Update the specified schema in sysrepo. Can be in either YANG or YIN format.\n"
-        "  -C, --connection-count\n"
-        "                       Print the number of sysrepo connections to STDOUT.\n"
-        "\n"
-        "Available other-options:\n"
-        "  -s, --search-dirs <dir-path>[:<dir-path>]*\n"
-        "                       Directories to search for include/import modules. Directory with already-installed\n"
-        "                       modules is always searched (install, update op).\n"
-        "  -e, --enable-feature <feature-name>\n"
-        "                       Enabled specific feature. Can be specified multiple times (install, change op).\n"
-        "  -d, --disable-feature <feature-name>\n"
-        "                       Disable specific feature. Can be specified multiple times (change op).\n"
-        "  -r, --replay <state> Change replay support (storing notifications) for this module to on/off or 1/0 (change op).\n"
-        "  -o, --owner <user>   Set filesystem owner of a module (change op, if \"apply\" even install, update op).\n"
-        "  -g, --group <group>  Set filesystem group of a module (change op, if \"apply\" even install, update op).\n"
-        "  -p, --permissions <permissions>\n"
-        "                       Set filesystem permissions of a module (chmod format) (change op, if \"apply\" even \n"
-        "                       install, update op).\n"
-        "  -a, --apply          Apply schema changes immediately, not only schedule them.\n"
-        "  -v, --verbosity <level>\n"
-        "                       Change verbosity to a level (none, error, warning, info, debug) or number (0, 1, 2, 3, 4).\n"
-        "\n"
-    );
+            "Usage:\n"
+            "  sysrepoctl <operation> [options]\n"
+            "\n"
+            "Available operations:\n"
+            "  -h, --help           Print usage help.\n"
+            "  -V, --version        Print only information about sysrepo version.\n"
+            "  -l, --list           List YANG modules in sysrepo.\n"
+            "  -i, --install <path> Install the specified schema into sysrepo. Can be in either YANG or YIN format.\n"
+            "  -u, --uninstall <module> [,<module2>,<module3>...]\n"
+            "                       Uninstall the specified module(s) from sysrepo.\n"
+            "  -c, --change <module>\n"
+            "                       Change access rights, features, or replay support of the specified module.\n"
+            "                       Use special \":ALL\" module name to change the access rights or replay support\n"
+            "                       of all the modules.\n"
+            "  -U, --update <path>  Update the specified schema in sysrepo. Can be in either YANG or YIN format.\n"
+            "  -C, --connection-count\n"
+            "                       Print the number of sysrepo connections to STDOUT.\n"
+            "\n"
+            "Available options:\n"
+            "  -s, --search-dirs <dir-path> [:<dir-path>...]\n"
+            "                       Directories to search for include/import modules. Directory with already-installed\n"
+            "                       modules is always searched. Accepted by install, update op.\n"
+            "  -e, --enable-feature <feature-name>\n"
+            "                       Enabled specific feature. Can be specified multiple times Accepted by install,\n"
+            "                       change op.\n"
+            "  -d, --disable-feature <feature-name>\n"
+            "                       Disable specific feature. Can be specified multiple times. Accepted by change op.\n"
+            "  -r, --replay <state> Change replay support (storing notifications) for this module to on/off or 1/0.\n"
+            "                       Accepted by change op.\n"
+            "  -o, --owner <user>   Set filesystem owner of a module. Accepted by change op, with \"apply\" even install,\n"
+            "                       update op.\n"
+            "  -g, --group <group>  Set filesystem group of a module. Accepted by change op, with \"apply\" even install,\n"
+            "                       update op.\n"
+            "  -p, --permissions <permissions>\n"
+            "                       Set filesystem permissions of a module (chmod format). Accepted by change op,\n"
+            "                       with \"apply\" even install, update op.\n"
+            "  -a, --apply          Apply schema changes immediately, not only schedule them. Accepted by install,\n"
+            "                       uninstall, change, update op.\n"
+            "  -v, --verbosity <level>\n"
+            "                       Change verbosity to a level (none, error, warning, info, debug) or\n"
+            "                       number (0, 1, 2, 3, 4). Accepted by all op.\n"
+            "\n");
 }
 
 static void
@@ -125,13 +126,17 @@ error_print(int sr_error, const char *format, ...)
     va_start(ap, format);
     vfprintf(stderr, msg, ap);
     va_end(ap);
+
+    if (log_level < SR_LL_INF) {
+        fprintf(stderr, "For more details you may try to increase the verbosity up to \"-v3\".\n");
+    }
 }
 
 static void
 srctl_list_collect_import(const struct lys_module *ly_mod, struct list_item **list, size_t *list_count)
 {
     struct list_item *cur_item;
-    uint32_t i;
+    LY_ARRAY_COUNT_TYPE u;
 
     if (ly_mod->implemented) {
         /* must be added from sysrepo data */
@@ -139,11 +144,11 @@ srctl_list_collect_import(const struct lys_module *ly_mod, struct list_item **li
     }
 
     /* check for duplicates */
-    for (i = 0; i < *list_count; ++i) {
-        if (strcmp((*list)[i].name, ly_mod->name)) {
+    for (u = 0; u < *list_count; ++u) {
+        if (strcmp((*list)[u].name, ly_mod->name)) {
             continue;
         }
-        if (strcmp((*list)[i].revision, ly_mod->rev_size ? ly_mod->rev[0].date : "")) {
+        if (strcmp((*list)[u].revision, ly_mod->revision ? ly_mod->revision : "")) {
             continue;
         }
 
@@ -165,11 +170,11 @@ srctl_list_collect_import(const struct lys_module *ly_mod, struct list_item **li
 
     /* fill name and revision */
     cur_item->name = strdup(ly_mod->name);
-    cur_item->revision = strdup(ly_mod->rev_size ? ly_mod->rev[0].date : "");
+    cur_item->revision = strdup(ly_mod->revision ? ly_mod->revision : "");
 
     /* recursively */
-    for (i = 0; i < ly_mod->imp_size; ++i) {
-        srctl_list_collect_import(ly_mod->imp[i].module, list, list_count);
+    LY_ARRAY_FOR(ly_mod->parsed->imports, u) {
+        srctl_list_collect_import(ly_mod->parsed->imports[u].module, list, list_count);
     }
 }
 
@@ -179,14 +184,19 @@ srctl_list_collect(sr_conn_ctx_t *conn, struct lyd_node *sr_data, const struct l
 {
     struct list_item *cur_item;
     const struct lys_module *ly_mod = NULL;
-    const struct lys_submodule *ly_submod;
+    const struct lysp_submodule *ly_submod;
     struct lyd_node *module, *child;
     char *owner, *group;
     const char *str;
-    int ret = SR_ERR_OK, real_state;
-    uint32_t i, j;
+    int ret = SR_ERR_OK;
+    LY_ERR real_state;
+    LY_ARRAY_COUNT_TYPE u, v;
+
+    LY_LIST_FOR(lyd_child(sr_data), module) {
+        if (strcmp(module->schema->name, "module") && strcmp(module->schema->name, "installed-module")) {
+            continue;
+        }
 
-    LY_TREE_FOR(sr_data->child, module) {
         /* new module */
         *list = realloc(*list, (*list_count + 1) * sizeof **list);
         cur_item = &(*list)[*list_count];
@@ -200,20 +210,22 @@ srctl_list_collect(sr_conn_ctx_t *conn, struct lyd_node *sr_data, const struct l
         cur_item->features = strdup("");
 
         /* name and revision must be first */
-        child = module->child;
+        child = lyd_child(module);
         assert(!strcmp(child->schema->name, "name"));
-        str = ((struct lyd_node_leaf_list *)child)->value_str;
-        cur_item->name = strdup(str);
+        cur_item->name = strdup(lyd_get_value(child));
 
         child = child->next;
         if (!strcmp(child->schema->name, "revision")) {
-            str = ((struct lyd_node_leaf_list *)child)->value_str;
-            cur_item->revision = strdup(str);
+            cur_item->revision = strdup(lyd_get_value(child));
         }
 
         /* get the module */
         if (!strcmp(module->schema->name, "module")) {
-            ly_mod = ly_ctx_get_module(ly_ctx, cur_item->name, cur_item->revision, 1);
+            if (cur_item->revision) {
+                ly_mod = ly_ctx_get_module(ly_ctx, cur_item->name, cur_item->revision);
+            } else {
+                ly_mod = ly_ctx_get_module_implemented(ly_ctx, cur_item->name);
+            }
             if (!ly_mod) {
                 return SR_ERR_INTERNAL;
             }
@@ -225,7 +237,7 @@ srctl_list_collect(sr_conn_ctx_t *conn, struct lyd_node *sr_data, const struct l
         }
 
         /* collect information from sysrepo data */
-        LY_TREE_FOR(child, child) {
+        LY_LIST_FOR(child, child) {
             if (!strcmp(child->schema->name, "replay-support")) {
                 cur_item->replay = 1;
             } else if (!strcmp(child->schema->name, "removed")) {
@@ -233,22 +245,22 @@ srctl_list_collect(sr_conn_ctx_t *conn, struct lyd_node *sr_data, const struct l
             } else if (!strcmp(child->schema->name, "updated-yang")) {
                 cur_item->change_flag = "U";
             } else if (!strcmp(child->schema->name, "enabled-feature")) {
-                str = ((struct lyd_node_leaf_list *)child)->value_str;
+                str = lyd_get_value(child);
 
                 if (ly_mod) {
                     /* check the real state of the feature */
-                    real_state = lys_features_state(ly_mod, str);
+                    real_state = lys_feature_value(ly_mod, str);
                 } else {
-                    real_state = 1;
+                    real_state = LY_SUCCESS;
                 }
 
-                cur_item->features = realloc(cur_item->features, strlen(cur_item->features) + (real_state ? 0 : 1) + 1
-                        + strlen(str) + 1);
+                cur_item->features = realloc(cur_item->features, strlen(cur_item->features) + (!real_state ? 0 : 1) + 1 +
+                        +strlen(str) + 1);
                 if (cur_item->features[0]) {
                     strcat(cur_item->features, " ");
                 }
                 strcat(cur_item->features, str);
-                if (!real_state) {
+                if (real_state == LY_ENOT) {
                     strcat(cur_item->features, "!");
                 }
             } else if (!strcmp(child->schema->name, "changed-feature")) {
@@ -269,26 +281,26 @@ srctl_list_collect(sr_conn_ctx_t *conn, struct lyd_node *sr_data, const struct l
             free(group);
 
             /* learn submodules */
-            for (i = 0; i < ly_mod->inc_size; ++i) {
-                str = ly_mod->inc[i].submodule->name;
+            LY_ARRAY_FOR(ly_mod->parsed->includes, u) {
+                str = ly_mod->parsed->includes[u].submodule->name;
                 cur_item->submodules = realloc(cur_item->submodules, strlen(cur_item->submodules) + 1 + strlen(str) + 1);
-                if (i) {
+                if (u) {
                     strcat(cur_item->submodules, " ");
                 }
                 strcat(cur_item->submodules, str);
             }
 
-            /* add all import modules of submodules as well */
-            for (i = 0; i < ly_mod->inc_size; ++i) {
-                ly_submod = ly_mod->inc[i].submodule;
-                for (j = 0; j < ly_submod->imp_size; ++j) {
-                    srctl_list_collect_import(ly_submod->imp[j].module, list, list_count);
+            /* add all import modules of submodules */
+            LY_ARRAY_FOR(ly_mod->parsed->includes, u) {
+                ly_submod = ly_mod->parsed->includes[u].submodule;
+                LY_ARRAY_FOR(ly_submod->imports, v) {
+                    srctl_list_collect_import(ly_submod->imports[v].module, list, list_count);
                 }
             }
 
             /* add all import modules as well */
-            for (i = 0; i < ly_mod->imp_size; ++i) {
-                srctl_list_collect_import(ly_mod->imp[i].module, list, list_count);
+            LY_ARRAY_FOR(ly_mod->parsed->imports, u) {
+                srctl_list_collect_import(ly_mod->parsed->imports[u].module, list, list_count);
             }
         } else {
             cur_item->change_flag = "N";
@@ -372,8 +384,8 @@ srctl_list(sr_conn_ctx_t *conn)
             max_feat_len, "Features");
 
     /* print ruler */
-    line_len = max_name_len + 3 + rev_len + 3 + flag_len + 3 + max_owner_len + 3 + perm_len + 3 + max_submod_len + 3
-            + max_feat_len;
+    line_len = max_name_len + 3 + rev_len + 3 + flag_len + 3 + max_owner_len + 3 + perm_len + 3 + max_submod_len + 3 +
+            max_feat_len;
     for (i = 0; i < line_len; ++i) {
         printf("-");
     }
@@ -389,8 +401,8 @@ srctl_list(sr_conn_ctx_t *conn)
             perm_str[0] = '\0';
         }
         printf("%-*s | %-*s | %-*s | %-*s | %-*s | %-*s | %-*s\n", max_name_len, list[i].name, rev_len, list[i].revision,
-            flag_len, flags_str, max_owner_len, list[i].owner, perm_len, perm_str, max_submod_len, list[i].submodules,
-            max_feat_len, list[i].features);
+                flag_len, flags_str, max_owner_len, list[i].owner, perm_len, perm_str, max_submod_len, list[i].submodules,
+                max_feat_len, list[i].features);
     }
 
     /* print flag legend */
@@ -398,7 +410,7 @@ srctl_list(sr_conn_ctx_t *conn)
     printf("Features: ! - Means that the feature is effectively disabled because of its false if-feature(s)\n\n");
 
 cleanup:
-    lyd_free_withsiblings(data);
+    lyd_free_all(data);
     for (i = 0; i < list_count; ++i) {
         free(list[i].name);
         free(list[i].revision);
@@ -438,13 +450,12 @@ log_cb(sr_log_level_t level, const char *message)
 }
 
 int
-main(int argc, char** argv)
+main(int argc, char **argv)
 {
     sr_conn_ctx_t *conn = NULL;
     const char *file_path = NULL, *search_dirs = NULL, *module_name = NULL, *owner = NULL, *group = NULL;
     char **features = NULL, **dis_features = NULL, *ptr;
     mode_t perms = -1;
-    sr_log_level_t log_level = SR_LL_ERR;
     int r, i, rc = EXIT_FAILURE, opt, operation = 0, feat_count = 0, dis_feat_count = 0, replay = -1, apply = 0;
     uint32_t conn_count;
     struct option options[] = {
@@ -455,7 +466,7 @@ main(int argc, char** argv)
         {"uninstall",       required_argument, NULL, 'u'},
         {"change",          required_argument, NULL, 'c'},
         {"update",          required_argument, NULL, 'U'},
-        {"connection-count",no_argument,       NULL, 'C'},
+        {"connection-count", no_argument,       NULL, 'C'},
         {"search-dirs",     required_argument, NULL, 's'},
         {"enable-feature",  required_argument, NULL, 'e'},
         {"disable-feature", required_argument, NULL, 'd'},
@@ -544,8 +555,9 @@ main(int argc, char** argv)
                 error_print(0, "Invalid parameter -%c for the operation", opt);
                 goto cleanup;
             }
-            features = realloc(features, (feat_count + 1) * sizeof *features);
+            features = realloc(features, (feat_count + 2) * sizeof *features);
             features[feat_count++] = optarg;
+            features[feat_count] = NULL;
             break;
         case 'd':
             if (operation && (operation != 'c')) {
@@ -596,7 +608,7 @@ main(int argc, char** argv)
                 error_print(0, "Invalid parameter -%c for the operation", opt);
                 goto cleanup;
             }
-            if ((int)perms != -1) {
+            if (perms != (mode_t)(-1)) {
                 error_print(0, "Permissions already specified");
                 goto cleanup;
             }
@@ -664,7 +676,7 @@ main(int argc, char** argv)
         break;
     case 'i':
         /* install */
-        if ((r = sr_install_module(conn, file_path, search_dirs, (const char **)features, feat_count)) != SR_ERR_OK) {
+        if ((r = sr_install_module(conn, file_path, search_dirs, (const char **)features)) != SR_ERR_OK) {
             /* succeed if the module is already installed */
             if (r != SR_ERR_EXISTS) {
                 error_print(r, "Failed to install module \"%s\"", file_path);
@@ -686,9 +698,17 @@ main(int argc, char** argv)
         /* change */
 
         /* change owner, group, and/or permissions */
-        if (owner || group || ((int)perms != -1)) {
+        if (owner || group || (perms != (mode_t)(-1))) {
+            if (!strcmp(module_name, ":ALL")) {
+                /* all the modules */
+                module_name = NULL;
+            }
             if ((r = sr_set_module_access(conn, module_name, owner, group, perms)) != SR_ERR_OK) {
-                error_print(r, "Failed to change module \"%s\" access", module_name);
+                if (module_name) {
+                    error_print(r, "Failed to change module \"%s\" access", module_name);
+                } else {
+                    error_print(r, "Failed to change modules access");
+                }
                 goto cleanup;
             }
         }
@@ -734,7 +754,7 @@ main(int argc, char** argv)
             error_print(r, "Failed to get connection count");
             goto cleanup;
         }
-        fprintf(stdout, "%u\n", conn_count);
+        fprintf(stdout, "%" PRIu32 "\n", conn_count);
         break;
     case 0:
         error_print(0, "No operation specified");
@@ -774,7 +794,7 @@ main(int argc, char** argv)
     }
 
     /* change permissions for a newly installed/updated module */
-    if (((operation == 'i') || (operation == 'U')) && (owner || group || ((int)perms != -1))) {
+    if (((operation == 'i') || (operation == 'U')) && (owner || group || (perms != (mode_t)(-1)))) {
         if ((r = sr_set_module_access(conn, inst_module_name, owner, group, perms)) != SR_ERR_OK) {
             error_print(r, "Failed to change module \"%s\" access", inst_module_name);
             goto cleanup;
