@@ -4,21 +4,19 @@
  * @brief Functions for simplified manipulation with Sysrepo values.
  *
  * @copyright
- * Copyright 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
-#define _POSIX_C_SOURCE 200809L
+
+#define _GNU_SOURCE
+
+#include "values.h"
 
 #include <inttypes.h>
 #include <stdarg.h>
@@ -26,7 +24,9 @@
 #include <unistd.h>
 
 #include "common.h"
-#include "values.h"
+#include "compat.h"
+#include "config.h"
+#include "log.h"
 
 /** get the larger item */
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -165,7 +165,7 @@ sr_mem_edit_string(char **string_p, const char *new_val)
 
     new_mem = strdup(new_val);
     if (!new_mem) {
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
 
     free(*string_p);
@@ -200,7 +200,7 @@ sr_mem_edit_string_va(char **string_p, const char *format, va_list args)
 
     new_mem = (char *)calloc(len + 1, sizeof(*new_mem));
     if (!new_mem) {
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
 
     vsnprintf(new_mem, len + 1, format, args);
@@ -402,7 +402,7 @@ sr_print(sr_print_ctx_t *print_ctx, const char *format, ...)
         len = vsnprintf(NULL, 0, format, va);
         str = calloc(len + 1, sizeof *str);
         if (!str) {
-            rc = SR_ERR_NOMEM;
+            rc = SR_ERR_NO_MEMORY;
             goto cleanup;
         }
         va_end(va);     /**< restart va_list */
@@ -417,7 +417,7 @@ sr_print(sr_print_ctx_t *print_ctx, const char *format, ...)
             new_size = MAX(2 * print_ctx->method.mem.size, print_ctx->method.mem.len + count + 1);
             aux = realloc(print_ctx->method.mem.buf, new_size * sizeof *aux);
             if (!aux) {
-                rc = SR_ERR_NOMEM;
+                rc = SR_ERR_NO_MEMORY;
                 goto cleanup;
             }
             print_ctx->method.mem.buf = aux;
@@ -597,7 +597,7 @@ sr_vasprintf(char **strp, const char *fmt, va_list ap)
     /* allocate memory for the string */
     buffer = calloc(size, sizeof *buffer);
     if (!buffer) {
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
 
     /* print */
@@ -848,17 +848,15 @@ sr_tree_to_val(const struct lyd_node *data, const char *path, sr_val_t **value)
 
     *value = NULL;
 
-    set = lyd_find_path(data, path);
-
-    if (!set) {
-        sr_errinfo_new_ly(&err_info, lyd_node_module(data)->ctx);
+    if (lyd_find_xpath(data, path, &set)) {
+        sr_errinfo_new_ly(&err_info, LYD_CTX(data));
         goto cleanup;
-    } else if (!set->number) {
+    } else if (!set->count) {
         /* Not building err_info to avoid error logs when no item found */
-        ly_set_free(set);
+        ly_set_free(set, NULL);
         return SR_ERR_NOT_FOUND;
-    } else if (set->number > 1) {
-        sr_errinfo_new(&err_info, SR_ERR_INVAL_ARG, NULL, "More subtrees match \"%s\".", path);
+    } else if (set->count > 1) {
+        sr_errinfo_new(&err_info, SR_ERR_INVAL_ARG, "More subtrees match \"%s\".", path);
         goto cleanup;
     }
 
@@ -866,13 +864,13 @@ sr_tree_to_val(const struct lyd_node *data, const char *path, sr_val_t **value)
     *value = malloc(sizeof **value);
     SR_CHECK_MEM_GOTO(!*value, err_info, cleanup);
 
-    if ((err_info = sr_val_ly2sr(set->set.d[0], *value))) {
+    if ((err_info = sr_val_ly2sr(set->dnodes[0], *value))) {
         goto cleanup;
     }
 
     /* success */
 cleanup:
-    ly_set_free(set);
+    ly_set_free(set, NULL);
     return sr_api_ret(NULL, err_info);
 }
 
@@ -888,25 +886,23 @@ sr_tree_to_values(const struct lyd_node *data, const char *xpath, sr_val_t **val
     *values = NULL;
     *value_cnt = 0;
 
-    set = lyd_find_path(data, xpath);
-
-    if (!set) {
-        sr_errinfo_new_ly(&err_info, lyd_node_module(data)->ctx);
+    if (lyd_find_xpath(data, xpath, &set)) {
+        sr_errinfo_new_ly(&err_info, LYD_CTX(data));
         goto cleanup;
-    } else if (!set->number) {
+    } else if (!set->count) {
         /* Not building err_info to avoid error logs when no item found */
-        ly_set_free(set);
+        ly_set_free(set, NULL);
         return SR_ERR_NOT_FOUND;
     } else {
-        *values = calloc(set->number, sizeof **values);
+        *values = calloc(set->count, sizeof **values);
         SR_CHECK_MEM_GOTO(!*values, err_info, cleanup);
 
-        for (i = 0; i < set->number; ++i) {
-            if (set->set.d[i]->schema->nodetype & (LYS_RPC | LYS_ACTION)) {
+        for (i = 0; i < set->count; ++i) {
+            if (set->dnodes[i]->schema->nodetype & (LYS_RPC | LYS_ACTION)) {
                 continue;
             }
 
-            if ((err_info = sr_val_ly2sr(set->set.d[i], *values + *value_cnt))) {
+            if ((err_info = sr_val_ly2sr(set->dnodes[i], *values + *value_cnt))) {
                 goto cleanup;
             }
             ++(*value_cnt);
@@ -915,7 +911,7 @@ sr_tree_to_values(const struct lyd_node *data, const char *xpath, sr_val_t **val
 
     /* success */
 cleanup:
-    ly_set_free(set);
+    ly_set_free(set, NULL);
     if (err_info) {
         sr_free_values(*values, *value_cnt);
         *values = NULL;
