@@ -4,34 +4,29 @@
  * @brief test for candidate datastore
  *
  * @copyright
- * Copyright 2018 Deutsche Telekom AG.
- * Copyright 2018 - 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE
 
-#include <string.h>
-#include <unistd.h>
 #include <setjmp.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
-#include "tests/config.h"
 #include "sysrepo.h"
+#include "tests/config.h"
 
 struct state {
     sr_conn_ctx_t *conn;
@@ -57,16 +52,16 @@ setup_f(void **state)
         return 1;
     }
 
-    if (sr_install_module(st->conn, TESTS_DIR "/files/test.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/test.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/ietf-interfaces.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/ietf-interfaces.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/iana-if-type.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/iana-if-type.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/when1.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/when1.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
     sr_disconnect(st->conn);
@@ -104,7 +99,7 @@ clear_interfaces(void **state)
 
     sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     sr_delete_item(st->sess, "/ietf-interfaces:interfaces", 0);
-    sr_apply_changes(st->sess, 0, 1);
+    sr_apply_changes(st->sess, 0);
 
     return 0;
 }
@@ -121,15 +116,15 @@ test_basic(void **state)
     /* empty datastore */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_CANDIDATE);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     /* modified running */
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
@@ -137,15 +132,15 @@ test_basic(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_CANDIDATE);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 0);
-    lyd_free_withsiblings(data);
+    assert_false(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     /* modify candidate */
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth32']/type",
@@ -153,7 +148,7 @@ test_basic(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_delete_item(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
@@ -161,8 +156,8 @@ test_basic(void **state)
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
         "<interface>"
@@ -178,8 +173,8 @@ test_basic(void **state)
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
         "<interface>"
@@ -198,14 +193,14 @@ test_basic(void **state)
     assert_int_equal(ret, SR_ERR_OK);
 
     /* copy-config to running, should also reset candidate */
-    ret = sr_copy_config(st->sess, NULL, SR_DS_CANDIDATE, 0, 1);
+    ret = sr_copy_config(st->sess, NULL, SR_DS_CANDIDATE, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
         "<interface>"
@@ -234,28 +229,28 @@ test_invalid(void **state)
     /* empty datastore */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_CANDIDATE);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     /* modify candidate */
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth32']", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check content */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
         "<interface>"
@@ -270,12 +265,12 @@ test_invalid(void **state)
     assert_int_equal(ret, SR_ERR_VALIDATION_FAILED);
 
     /* copy-config to candidate, should reset it */
-    ret = sr_copy_config(st->sess, NULL, SR_DS_RUNNING, 0, 1);
+    ret = sr_copy_config(st->sess, NULL, SR_DS_RUNNING, 0);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     assert_int_equal(ret, SR_ERR_OK);
 }
@@ -295,14 +290,14 @@ test_when(void **state)
     /* modify candidate */
     ret = sr_set_item_str(st->sess, "/when1:l3", "hi", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_data(st->sess, "/when1:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     assert_string_equal(str, str2);
     free(str);
 
@@ -312,7 +307,7 @@ test_when(void **state)
 
     ret = sr_delete_item(st->sess, "/when1:l3", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* should be valid again (empty) */
@@ -320,16 +315,16 @@ test_when(void **state)
     assert_int_equal(ret, SR_ERR_OK);
 
     /* the same change but with replace config */
-    data = lyd_parse_mem((struct ly_ctx *)sr_get_context(st->conn), str2, LYD_XML, LYD_OPT_CONFIG | LYD_OPT_TRUSTED | LYD_OPT_STRICT);
-    assert_non_null(data);
-    ret = sr_replace_config(st->sess, "when1", data, 0, 1);
+    assert_int_equal(LY_SUCCESS, lyd_parse_data_mem(sr_get_context(st->conn), str2, LYD_XML, LYD_PARSE_NO_STATE |
+            LYD_PARSE_ONLY | LYD_PARSE_STRICT, 0, &data));
+    ret = sr_replace_config(st->sess, "when1", data, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_data(st->sess, "/when1:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     assert_string_equal(str, str2);
     free(str);
 
@@ -340,13 +335,13 @@ test_when(void **state)
     /* copy-config to running, should fail */
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_copy_config(st->sess, NULL, SR_DS_CANDIDATE, 0, 1);
+    ret = sr_copy_config(st->sess, NULL, SR_DS_CANDIDATE, 0);
     assert_int_equal(ret, SR_ERR_VALIDATION_FAILED);
 
     /* copy-config to candidate, should reset it */
     ret = sr_session_switch_ds(st->sess, SR_DS_CANDIDATE);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_copy_config(st->sess, NULL, SR_DS_RUNNING, 0, 1);
+    ret = sr_copy_config(st->sess, NULL, SR_DS_RUNNING, 0);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     assert_int_equal(ret, SR_ERR_OK);
@@ -364,15 +359,15 @@ test_reset_unlock(void **state)
     /* empty datastore */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_CANDIDATE);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     /* lock candidate */
     ret = sr_lock(st->sess, NULL);
@@ -385,15 +380,15 @@ test_reset_unlock(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth32']/enabled",
             "false", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check content */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
         "<interface>"
@@ -412,8 +407,8 @@ test_reset_unlock(void **state)
     /* check content */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     assert_int_equal(ret, SR_ERR_OK);
@@ -436,15 +431,15 @@ test_reset_session_stop(void **state)
     /* empty datastore */
     ret = sr_get_data(sess2, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_CANDIDATE);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     /* lock candidate */
     ret = sr_lock(sess2, NULL);
@@ -457,15 +452,15 @@ test_reset_session_stop(void **state)
     ret = sr_set_item_str(sess2, "/ietf-interfaces:interfaces/interface[name='eth32']/enabled",
             "false", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(sess2, 0, 1);
+    ret = sr_apply_changes(sess2, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check content */
     ret = sr_get_data(sess2, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
         "<interface>"
@@ -484,8 +479,8 @@ test_reset_session_stop(void **state)
     /* check content */
     ret = sr_get_data(st->sess, "/ietf-interfaces:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     assert_int_equal(ret, SR_ERR_OK);
