@@ -4,34 +4,29 @@
  * @brief test for edits performed in a datastore
  *
  * @copyright
- * Copyright 2018 Deutsche Telekom AG.
- * Copyright 2018 - 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE
 
-#include <string.h>
-#include <unistd.h>
 #include <setjmp.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
-#include "tests/config.h"
 #include "sysrepo.h"
+#include "tests/config.h"
 
 struct state {
     sr_conn_ctx_t *conn;
@@ -57,28 +52,28 @@ setup_f(void **state)
         return 1;
     }
 
-    if (sr_install_module(st->conn, TESTS_DIR "/files/test.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/test.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/ietf-interfaces.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/ietf-interfaces.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/iana-if-type.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/iana-if-type.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/decimal.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/decimal.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/referenced-data.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/referenced-data.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/test-module.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/test-module.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/ops-ref.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/ops-ref.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/ops.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/ops.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
     sr_disconnect(st->conn);
@@ -119,7 +114,7 @@ clear_interfaces(void **state)
     struct state *st = (struct state *)*state;
 
     sr_delete_item(st->sess, "/ietf-interfaces:interfaces", 0);
-    sr_apply_changes(st->sess, 0, 1);
+    sr_apply_changes(st->sess, 0);
 
     return 0;
 }
@@ -133,7 +128,7 @@ clear_test(void **state)
     sr_delete_item(st->sess, "/test:ll1", 0);
     sr_delete_item(st->sess, "/test:cont", 0);
     sr_delete_item(st->sess, "/test:l3", 0);
-    sr_apply_changes(st->sess, 0, 1);
+    sr_apply_changes(st->sess, 0);
 
     return 0;
 }
@@ -174,7 +169,7 @@ test_edit_item(void **state)
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
-    /* whole edit should be removed */
+    /* user-ordered lists */
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
@@ -183,8 +178,10 @@ test_edit_item(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_move_item(st->sess, "/ietf-interfaces:interfaces/interface[name='eth2']",
             SR_MOVE_FIRST, NULL, NULL, NULL, 0);
-    assert_int_equal(ret, SR_ERR_OPERATION_FAILED);
-    assert_false(sr_has_changes(st->sess));
+    assert_int_equal(ret, SR_ERR_INVAL_ARG);
+    assert_true(sr_has_changes(st->sess));
+    ret = sr_discard_changes(st->sess);
+    assert_int_equal(ret, SR_ERR_OK);
 }
 
 static void
@@ -200,13 +197,13 @@ test_delete(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_delete_item(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* delete on no data */
     ret = sr_delete_item(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type", SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_NOT_FOUND);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
@@ -214,21 +211,21 @@ test_delete(void **state)
     /* delete a leaf without exact value */
     ret = sr_set_item_str(st->sess, "/test:test-leaf", "16", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_delete_item(st->sess, "/test:test-leaf", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check final datastore contents */
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS);
     assert_null(str);
-    lyd_free(subtree);
+    lyd_free_tree(subtree);
 }
 
 static void
@@ -246,14 +243,14 @@ test_create1(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free(subtree);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_tree(subtree);
 
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
@@ -268,14 +265,14 @@ test_create1(void **state)
 
     ret = sr_delete_item(st->sess, "/ietf-interfaces:interfaces", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* create with non-existing parents */
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_NON_RECURSIVE);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_NOT_FOUND);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
@@ -295,14 +292,14 @@ test_create2(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free(subtree);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_tree(subtree);
 
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
@@ -317,15 +314,15 @@ test_create2(void **state)
 
     ret = sr_delete_item(st->sess, "/ietf-interfaces:interfaces", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS);
     assert_null(str);
-    lyd_free(subtree);
+    lyd_free_tree(subtree);
 }
 
 static void
@@ -339,29 +336,29 @@ test_create_np_cont(void **state)
     assert_int_equal(ret, SR_ERR_OK);
 
     assert_string_equal(subtree->schema->name, "interfaces");
-    assert_int_equal(subtree->dflt, 1);
-    lyd_free(subtree);
+    assert_true(subtree->flags & LYD_DEFAULT);
+    lyd_free_tree(subtree);
 
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
     assert_string_equal(subtree->schema->name, "interfaces");
-    assert_int_equal(subtree->dflt, 1);
-    lyd_free(subtree);
+    assert_true(subtree->flags & LYD_DEFAULT);
+    lyd_free_tree(subtree);
 
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 }
 
@@ -369,9 +366,8 @@ static void
 test_move(void **state)
 {
     struct state *st = (struct state *)*state;
-    struct lyd_node *data, *node;
+    struct lyd_node *data;
     char *str, *str2;
-    uint32_t i;
     int ret;
 
     /* create top-level testing data */
@@ -389,7 +385,7 @@ test_move(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/test:ll1", "-3", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* perform some move operations */
@@ -401,59 +397,35 @@ test_move(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_move_item(st->sess, "/test:ll1[.='-1']", SR_MOVE_AFTER, NULL, "-2", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_data(st->sess, "/test:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    /* should be in reversed order (relative only to the same schema node instances) */
-    for (node = data, i = 0; i < 7; ++i, node = node->next) {
-        lyd_print_mem(&str, node, LYD_XML, 0);
+    /* should be in reversed order */
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
 
-        switch (i) {
-        case 0:
-            asprintf(&str2, "<ll1 xmlns=\"urn:test\">-%u</ll1>", 3);
-            break;
-        case 1:
-            asprintf(&str2,
-            "<l1 xmlns=\"urn:test\">"
-                "<k>key%u</k>"
-                "<v>%u</v>"
-            "</l1>", 3, 3);
-            break;
-        case 2:
-            assert_null(str);
-            continue;
-        case 3:
-            asprintf(&str2,
-            "<l1 xmlns=\"urn:test\">"
-                "<k>key%u</k>"
-                "<v>%u</v>"
-            "</l1>", 2, 2);
-            break;
-        case 4:
-            asprintf(&str2,
-            "<l1 xmlns=\"urn:test\">"
-                "<k>key%u</k>"
-                "<v>%u</v>"
-            "</l1>", 1, 1);
-            break;
-        case 5:
-            asprintf(&str2, "<ll1 xmlns=\"urn:test\">-%u</ll1>", 2);
-            break;
-        case 6:
-            asprintf(&str2, "<ll1 xmlns=\"urn:test\">-%u</ll1>", 1);
-            break;
-        default:
-            fail();
-        }
+    str2 =
+    "<l1 xmlns=\"urn:test\">"
+        "<k>key3</k>"
+        "<v>3</v>"
+    "</l1>"
+    "<l1 xmlns=\"urn:test\">"
+        "<k>key2</k>"
+        "<v>2</v>"
+    "</l1>"
+    "<l1 xmlns=\"urn:test\">"
+        "<k>key1</k>"
+        "<v>1</v>"
+    "</l1>"
+    "<ll1 xmlns=\"urn:test\">-3</ll1>"
+    "<ll1 xmlns=\"urn:test\">-2</ll1>"
+    "<ll1 xmlns=\"urn:test\">-1</ll1>";
+    assert_string_equal(str, str2);
 
-        assert_string_equal(str, str2);
-        free(str2);
-        free(str);
-    }
-    lyd_free_withsiblings(data);
+    free(str);
+    lyd_free_all(data);
 
     /* create nested testing data */
     ret = sr_set_item_str(st->sess, "/test:cont/l2[k='key1']/v", "1", NULL, 0);
@@ -468,7 +440,7 @@ test_move(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/test:cont/ll2", "-3", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* perform some move operations */
@@ -480,28 +452,28 @@ test_move(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_move_item(st->sess, "/test:cont/ll2[.='-3']", SR_MOVE_BEFORE, NULL, "-2", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_data(st->sess, "/test:cont", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    /* should be in reversed order (relative only to the same schema node instances) */
-    lyd_print_mem(&str, data, LYD_XML, 0);
+    /* should be in reversed order */
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_SHRINK);
 
     str2 =
     "<cont xmlns=\"urn:test\">"
         "<l2><k>key3</k><v>3</v></l2>"
         "<l2><k>key2</k><v>2</v></l2>"
+        "<l2><k>key1</k><v>1</v></l2>"
         "<ll2>-3</ll2>"
         "<ll2>-2</ll2>"
-        "<l2><k>key1</k><v>1</v></l2>"
         "<ll2>-1</ll2>"
     "</cont>";
     assert_string_equal(str, str2);
 
     free(str);
-    lyd_free_withsiblings(data);
+    lyd_free_all(data);
 }
 
 static void
@@ -518,7 +490,7 @@ test_replace(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* remove and create some other data, internally transformed into replace */
@@ -529,15 +501,15 @@ test_replace(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth32']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check final datastore contents */
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free(subtree);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_tree(subtree);
 
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
@@ -562,7 +534,7 @@ test_replace_userord(void **state)
     /* create some data */
     ret = sr_set_item_str(st->sess, "/test:l3[k='one']", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* replace some data with a custom edit */
@@ -571,20 +543,19 @@ test_replace_userord(void **state)
         "<k>one</k>"
         "<ll3>3</ll3>"
     "</l3>";
-    edit = lyd_parse_mem((struct ly_ctx *)sr_get_context(st->conn), str2, LYD_XML, LYD_OPT_EDIT);
-    assert_non_null(edit);
+    assert_int_equal(LY_SUCCESS, lyd_parse_data_mem(sr_get_context(st->conn), str2, LYD_XML, LYD_PARSE_ONLY, 0, &edit));
     ret = sr_edit_batch(st->sess, edit, "merge");
-    lyd_free_withsiblings(edit);
+    lyd_free_all(edit);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check final datastore contents */
     ret = sr_get_data(st->sess, "/test:*", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free_withsiblings(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_all(data);
 
     str2 =
     "<l3 xmlns=\"urn:test\">"
@@ -611,7 +582,7 @@ test_isolate(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth64']/type",
             "iana-if-type:softwareLoopback", NULL, SR_EDIT_STRICT | SR_EDIT_ISOLATE);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_EXISTS);
 
     ret = sr_discard_changes(st->sess);
@@ -628,15 +599,15 @@ test_isolate(void **state)
     ret = sr_move_item(st->sess, "/ietf-interfaces:interfaces/interface[name='eth32']",
             SR_MOVE_FIRST, NULL, NULL, NULL, SR_EDIT_ISOLATE);
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check datastore contents */
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free(subtree);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_tree(subtree);
 
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
@@ -662,15 +633,15 @@ test_isolate(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_delete_item(st->sess, "/test:test-leaf", SR_EDIT_STRICT | SR_EDIT_ISOLATE);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check datastore contents */
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free(subtree);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_tree(subtree);
 
     str2 =
     "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\">"
@@ -704,20 +675,20 @@ test_purge(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth67']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* delete all instances */
     ret = sr_delete_item(st->sess, "/ietf-interfaces:interfaces/interface", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check datastore contents */
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(subtree->dflt, 1);
-    lyd_free(subtree);
+    assert_true(subtree->flags & LYD_DEFAULT);
+    lyd_free_tree(subtree);
 
     /* repeat with leaf-list */
     ret = sr_set_item_str(st->sess, "/test:ll1", "12", NULL, SR_EDIT_STRICT);
@@ -726,12 +697,12 @@ test_purge(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/test:ll1", "14", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_delete_item(st->sess, "/test:ll1", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_get_subtree(st->sess, "/test:ll1", 0, &subtree);
@@ -758,26 +729,26 @@ test_top_op(void **state)
     ret = sr_set_item_str(st->sess, "/ietf-interfaces:interfaces/interface[name='eth65']/type",
             "iana-if-type:ethernetCsmacd", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* replace the top-level container with an empty one */
     str = "<interfaces xmlns=\"urn:ietf:params:xml:ns:yang:ietf-interfaces\" nc:operation=\"replace\" xmlns:nc=\"urn:ietf:params:xml:ns:netconf:base:1.0\"/>";
-    data = lyd_parse_mem((struct ly_ctx *)sr_get_context(st->conn), str, LYD_XML, LYD_OPT_EDIT | LYD_OPT_STRICT);
-    assert_non_null(data);
+    assert_int_equal(LY_SUCCESS, lyd_parse_data_mem(sr_get_context(st->conn), str, LYD_XML,
+            LYD_PARSE_ONLY | LYD_PARSE_STRICT, 0, &data));
 
     ret = sr_edit_batch(st->sess, data, "merge");
-    lyd_free_withsiblings(data);
+    lyd_free_all(data);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check datastore contents */
     ret = sr_get_subtree(st->sess, "/ietf-interfaces:interfaces", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    assert_int_equal(data->dflt, 1);
-    lyd_free_withsiblings(data);
+    assert_true(data->flags & LYD_DEFAULT);
+    lyd_free_all(data);
 }
 
 static void
@@ -792,21 +763,21 @@ test_union(void **state)
     /* create some host */
     ret = sr_set_item_str(st->sess, "/test:cont/server", "fe80::42:39ff:fe67:1fb3", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* change it */
     ret = sr_set_item_str(st->sess, "/test:cont/server", "192.168.1.10", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* check datastore contents */
     ret = sr_get_subtree(st->sess, "/test:cont", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
 
-    lyd_print_mem(&str, data, LYD_XML, LYP_WITHSIBLINGS);
-    lyd_free(data);
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
+    lyd_free_tree(data);
 
     str2 =
     "<cont xmlns=\"urn:test\">"
@@ -838,8 +809,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d1", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "255.5");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "255.5");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -850,8 +821,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d-uni-2-18", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "9.0000000000000001");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "9.0000000000000001");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -859,8 +830,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d-uni-2-18", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "2.01");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "2.01");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -878,8 +849,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d1", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "255.6");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "255.6");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -889,8 +860,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d-uni-2-18", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "10.0");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "10.0");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -900,8 +871,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d-uni-2-18", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "9.0");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "9.0");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -911,8 +882,8 @@ test_decimal64(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/decimal:d-uni-2-18", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "2.01");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "2.01");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 }
@@ -924,7 +895,7 @@ test_mutiple_types(void **state)
     struct lyd_node *data;
     char *str, *str2;
     int ret;
-    sr_val_t val;
+    sr_val_t val = {0};
 
     /* type string */
     val.type = SR_STRING_T;
@@ -933,8 +904,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/string", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "string\"\"\'");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "string\"\"\'");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -945,8 +916,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/raw", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "VGhpcyBpcyBleGFtcGxlIG1lc3NhZ2Uu");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "VGhpcyBpcyBleGFtcGxlIG1lc3NhZ2Uu");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -957,8 +928,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/options", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "strict");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "strict");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -969,8 +940,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/enum", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "yes");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "yes");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -981,8 +952,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/id_ref", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "test-module:id_1");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "test-module:id_1");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -993,15 +964,15 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_data(st->sess, "/test-module:main/instance_id", 0, 0, 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    lyd_print_mem(&str, data, LYD_XML, 0);
-    asprintf(&str2,
+    lyd_print_mem(&str, data, LYD_XML, LYD_PRINT_SHRINK);
+    assert_return_code(asprintf(&str2,
     "<main xmlns=\"urn:ietf:params:xml:ns:yang:test-module\">"
         "<instance_id xmlns:tm=\"urn:ietf:params:xml:ns:yang:test-module\">/tm:main/tm:options</instance_id>"
-    "</main>");
+    "</main>"), 0);
     assert_string_equal(str, str2);
     free(str);
     free(str2);
-    lyd_free(data);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1012,8 +983,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/any-data", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_anydata *)data)->value.str, "test");
-    lyd_free(data);
+    assert_string_equal(((struct lyd_node_any *)data)->value.str, "test");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1023,8 +994,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/empty", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_string_equal(((struct lyd_node_leaf_list *)data)->value_str, "");
-    lyd_free(data);
+    assert_string_equal(lyd_get_value(data), "");
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1035,8 +1006,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/boolean", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.bln, 1);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.boolean, 1);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1047,8 +1018,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/ui8", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.uint8, UINT8_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.uint8, UINT8_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1059,8 +1030,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/ui16", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.uint16, UINT16_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.uint16, UINT16_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1071,8 +1042,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/ui32", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.uint32, UINT32_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.uint32, UINT32_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1083,8 +1054,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/ui64", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.uint64, UINT64_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.uint64, UINT64_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1095,8 +1066,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/i8", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.int8, INT8_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.int8, INT8_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1107,8 +1078,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/i16", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.int16, INT16_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.int16, INT16_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1119,8 +1090,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/i32", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.int32, INT32_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.int32, INT32_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 
@@ -1131,8 +1102,8 @@ test_mutiple_types(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/test-module:main/i64", 0, &data);
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(((struct lyd_node_leaf_list *)data)->value.int64, INT64_MAX);
-    lyd_free(data);
+    assert_int_equal(((struct lyd_node_term *)data)->value.int64, INT64_MAX);
+    lyd_free_tree(data);
     ret = sr_discard_changes(st->sess);
     assert_int_equal(ret, SR_ERR_OK);
 }
@@ -1148,15 +1119,15 @@ test_edit_forbid_node_types(void **state)
     int ret;
 
     /* set some data needed for validation */
-    ret=sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, SR_EDIT_STRICT);
+    ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* rpc node cannot be created */
     ret = sr_set_item_str(st->sess, "/ops:rpc3/l4", "val", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/ops:rpc3/l4", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
@@ -1165,7 +1136,7 @@ test_edit_forbid_node_types(void **state)
     /* action node cannot be created */
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='k']/cont2/act1/l6", "val", NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/ops:cont/list1[k='k']/cont2/act1/l6", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
@@ -1174,7 +1145,7 @@ test_edit_forbid_node_types(void **state)
     /* notification node cannot be created */
     ret = sr_set_item_str(st->sess, "/ops:notif3/list2[k='k']", NULL, NULL, SR_EDIT_STRICT);
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
-    ret = sr_apply_changes(st->sess, 0, 1);
+    ret = sr_apply_changes(st->sess, 0);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_get_subtree(st->sess, "/ops:notif3/list2[k='k']", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
@@ -1183,7 +1154,7 @@ test_edit_forbid_node_types(void **state)
     /* not throw away the whole edit, the successfully created node still exists */
     ret = sr_get_subtree(st->sess, "/ops:cont", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
-    lyd_print_mem(&str, subtree, LYD_XML, LYP_WITHSIBLINGS);
+    lyd_print_mem(&str, subtree, LYD_XML, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK);
 
     str2 =
     "<cont xmlns=\"urn:ops\">"
@@ -1193,7 +1164,7 @@ test_edit_forbid_node_types(void **state)
     "</cont>";
 
     assert_string_equal(str, str2);
-    lyd_free(subtree);
+    lyd_free_tree(subtree);
     free(str);
 }
 
