@@ -4,37 +4,33 @@
  * @brief test for sysrepo API and internal mod locks
  *
  * @copyright
- * Copyright 2018 Deutsche Telekom AG.
- * Copyright 2018 - 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE
 
-#include <sys/types.h>
-#include <unistd.h>
 #include <pthread.h>
-#include <stdlib.h>
 #include <setjmp.h>
-#include <string.h>
-#include <stdlib.h>
 #include <stdarg.h>
+#include <stdlib.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
-#include "tests/config.h"
 #include "sysrepo.h"
+#include "tests/config.h"
 
 struct state {
     sr_conn_ctx_t *conn;
@@ -56,19 +52,19 @@ setup(void **state)
         return 1;
     }
 
-    if (sr_install_module(st->conn, TESTS_DIR "/files/test.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/test.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/ietf-interfaces.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/ietf-interfaces.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/iana-if-type.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/iana-if-type.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/when1.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/when1.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
-    if (sr_install_module(st->conn, TESTS_DIR "/files/when2.yang", TESTS_DIR "/files", NULL, 0) != SR_ERR_OK) {
+    if (sr_install_module(st->conn, TESTS_SRC_DIR "/files/when2.yang", TESTS_SRC_DIR "/files", NULL) != SR_ERR_OK) {
         return 1;
     }
     sr_disconnect(st->conn);
@@ -285,7 +281,7 @@ test_session_stop_unlock(void **state)
     /* read some data while the module is locked */
     ret = sr_get_subtree(sess2, "/test:cont", 0, &subtree);
     assert_int_equal(ret, SR_ERR_OK);
-    lyd_free_withsiblings(subtree);
+    lyd_free_tree(subtree);
 
     /* stop session with locks */
     sr_session_stop(sess1);
@@ -308,14 +304,14 @@ test_get_lock(void **state)
     struct state *st = (struct state *)*state;
     sr_session_ctx_t *sess;
     int ret, is_locked;
-    uint32_t id, nc_id;
-    time_t timestamp;
+    uint32_t id;
+    struct timespec timestamp;
 
     /* params error, connection null or datastore is operational */
-    ret=sr_get_lock(NULL, SR_DS_RUNNING, NULL, &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(NULL, SR_DS_RUNNING, NULL, &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
 
-    ret = sr_get_lock(st->conn, SR_DS_OPERATIONAL, NULL, &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(st->conn, SR_DS_OPERATIONAL, NULL, &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
 
     ret = sr_session_start(st->conn, SR_DS_RUNNING, &sess);
@@ -326,12 +322,12 @@ test_get_lock(void **state)
     assert_int_equal(ret, SR_ERR_OK);
 
     /* get lock for a locked module */
-    ret = sr_get_lock(st->conn, SR_DS_RUNNING, "test", &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(st->conn, SR_DS_RUNNING, "test", &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_OK);
     assert_int_equal(is_locked, 1);
 
     /* get lock for all modules */
-    ret = sr_get_lock(st->conn, SR_DS_RUNNING, NULL, &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(st->conn, SR_DS_RUNNING, NULL, &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_OK);
     assert_int_equal(is_locked, 0);
 
@@ -340,7 +336,7 @@ test_get_lock(void **state)
     assert_int_equal(ret, SR_ERR_OK);
 
     /* get lock for a unlocked module */
-    ret = sr_get_lock(st->conn, SR_DS_RUNNING, "test", &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(st->conn, SR_DS_RUNNING, "test", &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_OK);
     assert_int_equal(is_locked, 0);
 
@@ -349,12 +345,12 @@ test_get_lock(void **state)
     assert_int_equal(ret, SR_ERR_OK);
 
     /* get lock for all modules */
-    ret = sr_get_lock(st->conn, SR_DS_RUNNING, NULL, &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(st->conn, SR_DS_RUNNING, NULL, &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_OK);
     assert_int_equal(is_locked, 1);
 
     /* get lock for another module */
-    ret = sr_get_lock(st->conn, SR_DS_RUNNING, "when1", &is_locked, &id, &nc_id, &timestamp);
+    ret = sr_get_lock(st->conn, SR_DS_RUNNING, "when1", &is_locked, &id, &timestamp);
     assert_int_equal(ret, SR_ERR_OK);
     assert_int_equal(is_locked, 1);
 
