@@ -4,30 +4,25 @@
  * @brief test for concurrent execution of several sysrepo processes
  *
  * @copyright
- * Copyright 2020 Deutsche Telekom AG.
- * Copyright 2020 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 
-#include "tests/config.h"
 #include "sysrepo.h"
+#include "tests/config.h"
 
 #define sr_assert(cond) if (!(cond)) { fprintf(stderr, "\"%s\" not true\n", #cond); sr_assert_line(); abort(); }
 
@@ -83,8 +78,8 @@ run_tests(struct test *tests, uint32_t test_count)
     const char *child_status, *parent_status;
     size_t i;
 
-    pipe(pipes);
-    pipe(pipes + 2);
+    sr_assert(pipe(pipes) == 0);
+    sr_assert(pipe(pipes + 2) == 0);
 
     printf("[===========] Running %u test(s).\n", test_count);
 
@@ -177,14 +172,14 @@ static void
 setup(void)
 {
     sr_conn_ctx_t *conn;
-    const char *en_feat = "feat1";
+    const char *en_feats[] = {"feat1", NULL};
 
     sr_assert(sr_connect(0, &conn) == SR_ERR_OK);
 
-    sr_assert(sr_install_module(conn, TESTS_DIR "/files/ops-ref.yang", TESTS_DIR "/files", &en_feat, 1) == SR_ERR_OK);
-    sr_assert(sr_install_module(conn, TESTS_DIR "/files/ops.yang", TESTS_DIR "/files", NULL, 0) == SR_ERR_OK);
-    sr_assert(sr_install_module(conn, TESTS_DIR "/files/ietf-interfaces.yang", TESTS_DIR "/files", NULL, 0) == SR_ERR_OK);
-    sr_assert(sr_install_module(conn, TESTS_DIR "/files/iana-if-type.yang", TESTS_DIR "/files", NULL, 0) == SR_ERR_OK);
+    sr_assert(sr_install_module(conn, TESTS_SRC_DIR "/files/ops-ref.yang", TESTS_SRC_DIR "/files", en_feats) == SR_ERR_OK);
+    sr_assert(sr_install_module(conn, TESTS_SRC_DIR "/files/ops.yang", TESTS_SRC_DIR "/files", NULL) == SR_ERR_OK);
+    sr_assert(sr_install_module(conn, TESTS_SRC_DIR "/files/ietf-interfaces.yang", TESTS_SRC_DIR "/files", NULL) == SR_ERR_OK);
+    sr_assert(sr_install_module(conn, TESTS_SRC_DIR "/files/iana-if-type.yang", TESTS_SRC_DIR "/files", NULL) == SR_ERR_OK);
 
     sr_disconnect(conn);
 }
@@ -206,10 +201,11 @@ teardown(void)
 
 /* TEST */
 static int
-rpc_sub_cb(sr_session_ctx_t *session, const char *op_path, const sr_val_t *input, const size_t input_cnt,
+rpc_sub_cb(sr_session_ctx_t *session, uint32_t sub_id, const char *op_path, const sr_val_t *input, const size_t input_cnt,
         sr_event_t event, uint32_t request_id, sr_val_t **output, size_t *output_cnt, void *private_data)
 {
     (void)session;
+    (void)sub_id;
     (void)op_path;
     (void)input;
     (void)input_cnt;
@@ -318,23 +314,23 @@ test_rpc_crash1(int rp, int wp)
     /* wait for the other process */
     barrier(rp, wp);
 
-    rpc = lyd_new_path(NULL, sr_get_context(conn), "/ops:rpc3/l4", "value", 0, 0);
-    sr_assert_true(rpc);
+    sr_assert_int_equal(LY_SUCCESS, lyd_new_path(NULL, sr_get_context(conn), "/ops:rpc3/l4", "value", 0, &rpc));
 
     /* this should crash the other process */
     ret = sr_rpc_send_tree(sess, rpc, 2000, &output);
     sr_assert_int_equal(ret, SR_ERR_CALLBACK_FAILED);
 
-    lyd_free(rpc);
+    lyd_free_tree(rpc);
     sr_disconnect(conn);
     return 0;
 }
 
 static int
-rpc_crash_cb(sr_session_ctx_t *session, const char *op_path, const sr_val_t *input, const size_t input_cnt,
+rpc_crash_cb(sr_session_ctx_t *session, uint32_t sub_id, const char *op_path, const sr_val_t *input, const size_t input_cnt,
         sr_event_t event, uint32_t request_id, sr_val_t **output, size_t *output_cnt, void *private_data)
 {
     (void)session;
+    (void)sub_id;
     (void)op_path;
     (void)input;
     (void)input_cnt;
@@ -344,6 +340,9 @@ rpc_crash_cb(sr_session_ctx_t *session, const char *op_path, const sr_val_t *inp
     (void)output_cnt;
     (void)private_data;
 
+    /* avoid leaks (valgrind probably cannot keep track of leafref attributes because they are shared) */
+    ly_ctx_destroy((struct ly_ctx *)sr_get_context(sr_session_get_connection(session)));
+
     /* callback crashes */
     exit(0);
 
@@ -384,11 +383,13 @@ test_rpc_crash2(int rp, int wp)
 
 /* TEST */
 static void
-notif_instid_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const char *xpath,
-        const sr_val_t *values, const size_t values_cnt, time_t timestamp, void *private_data)
+notif_instid_cb(sr_session_ctx_t *session, uint32_t sub_id, const sr_ev_notif_type_t notif_type, const char *xpath,
+        const sr_val_t *values, const size_t values_cnt, struct timespec *timestamp, void *private_data)
 {
     (void)session;
+    (void)sub_id;
     (void)notif_type;
+    (void)sub_id;
     (void)xpath;
     (void)values;
     (void)values_cnt;
@@ -397,10 +398,11 @@ notif_instid_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type,
 }
 
 static int
-notif_instid_change_cb(sr_session_ctx_t *session, const char *module_name, const char *xpath,
+notif_instid_change_cb(sr_session_ctx_t *session, uint32_t sub_id, const char *module_name, const char *xpath,
         sr_event_t event, uint32_t request_id, void *private_data)
 {
     (void)session;
+    (void)sub_id;
     (void)module_name;
     (void)xpath;
     (void)event;
@@ -429,7 +431,7 @@ test_notif_instid1(int rp, int wp)
     ret = sr_set_item_str(sess, "/ietf-interfaces:interfaces/interface[name='eth0']/type", "iana-if-type:ethernetCsmacd",
             NULL, 0);
     sr_assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(sess, 0, 1);
+    ret = sr_apply_changes(sess, 0);
     sr_assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to it so it appears in operational */
@@ -441,8 +443,8 @@ test_notif_instid1(int rp, int wp)
     sr_assert_int_equal(ret, SR_ERR_OK);
 
     /* create the notification */
-    notif = lyd_new_path(NULL, sr_get_context(conn), "/ops:notif3/list2[k='key']/l15",
-            "/ietf-interfaces:interfaces/interface[name='eth0']", 0, 0);
+    lyd_new_path(NULL, sr_get_context(conn), "/ops:notif3/list2[k='key']/l15",
+            "/ietf-interfaces:interfaces/interface[name='eth0']", 0, &notif);
     sr_assert(notif);
 
     /* wait for the other process */
@@ -450,11 +452,11 @@ test_notif_instid1(int rp, int wp)
 
     /* keep sending notification with instance-identifier */
     for (i = 0; i < 50; ++i) {
-        ret = sr_event_notif_send_tree(sess, notif);
+        ret = sr_event_notif_send_tree(sess, notif, 0, 0);
         sr_assert_int_equal(ret, SR_ERR_OK);
     }
 
-    lyd_free(notif);
+    lyd_free_tree(notif);
 
     sr_unsubscribe(sub);
     sr_disconnect(conn);
@@ -483,14 +485,14 @@ test_notif_instid2(int rp, int wp)
         sr_assert_int_equal(ret, SR_ERR_OK);
         ret = sr_set_item_str(sess, "/ietf-interfaces:interfaces/interface[name='eth0']/description", "desc", NULL, 0);
         sr_assert_int_equal(ret, SR_ERR_OK);
-        ret = sr_apply_changes(sess, 0, 1);
+        ret = sr_apply_changes(sess, 0);
         sr_assert_int_equal(ret, SR_ERR_OK);
 
         ret = sr_set_item_str(sess, "/ietf-interfaces:interfaces/interface[name='eth0']/enabled", "true", NULL, 0);
         sr_assert_int_equal(ret, SR_ERR_OK);
         ret = sr_set_item_str(sess, "/ietf-interfaces:interfaces/interface[name='eth0']/description", "desc2", NULL, 0);
         sr_assert_int_equal(ret, SR_ERR_OK);
-        ret = sr_apply_changes(sess, 0, 1);
+        ret = sr_apply_changes(sess, 0);
         sr_assert_int_equal(ret, SR_ERR_OK);
     }
 
@@ -498,6 +500,122 @@ test_notif_instid2(int rp, int wp)
     return 0;
 }
 
+/* TEST */
+static int
+pull_oper_cb(sr_session_ctx_t *session, uint32_t sub_id, const char *module_name, const char *path,
+        const char *request_xpath, uint32_t request_id, struct lyd_node **parent, void *private_data)
+{
+    const struct ly_ctx *ly_ctx = sr_get_context(sr_session_get_connection(session));
+    LY_ERR ret;
+
+    (void)sub_id;
+    (void)module_name;
+    (void)path;
+    (void)request_xpath;
+    (void)request_id;
+    (void)private_data;
+
+    ret = lyd_new_path(NULL, ly_ctx, "/ietf-interfaces:interfaces-state/interface[name='lo']/statistics/discontinuity-time",
+            "2021-03-01T00:00:00Z", 0, parent);
+    sr_assert_int_equal(ret, LY_SUCCESS);
+    ret = lyd_new_path(*parent, NULL, "/ietf-interfaces:interfaces-state/interface[name='lo']/statistics/in-octets",
+            "42", 0, NULL);
+    sr_assert_int_equal(ret, LY_SUCCESS);
+    ret = lyd_new_path(*parent, NULL, "/ietf-interfaces:interfaces-state/interface[name='lo']/statistics/out-octets",
+            "42", 0, NULL);
+    sr_assert_int_equal(ret, LY_SUCCESS);
+
+    return SR_ERR_OK;
+}
+
+static int
+test_pull_push_oper1(int rp, int wp)
+{
+    sr_conn_ctx_t *conn;
+    sr_session_ctx_t *sess;
+    sr_subscription_ctx_t *sub;
+    int ret, i;
+
+    ret = sr_connect(0, &conn);
+    sr_assert_int_equal(ret, SR_ERR_OK);
+
+    ret = sr_session_start(conn, SR_DS_OPERATIONAL, &sess);
+    sr_assert_int_equal(ret, SR_ERR_OK);
+
+    /* subscribe to providing operational data */
+    ret = sr_oper_get_items_subscribe(sess, "ietf-interfaces", "/ietf-interfaces:interfaces-state/interface/statistics",
+            pull_oper_cb, NULL, 0, &sub);
+    sr_assert_int_equal(ret, SR_ERR_OK);
+
+    /* wait for the other process */
+    barrier(rp, wp);
+
+    /* keep changing push operational data */
+    for (i = 0; i < 200; ++i) {
+        ret = sr_set_item_str(sess, "/ietf-interfaces:interfaces-state/interface[name='lo']/phys-address",
+                (i % 2) ? "00:00:00:00:00:00" : "11:11:11:11:11:11", NULL, 0);
+        sr_assert_int_equal(ret, SR_ERR_OK);
+        ret = sr_set_item_str(sess, "/ietf-interfaces:interfaces-state/interface[name='lo']/oper-status",
+                (i % 2) ? "unknown" : "down", NULL, 0);
+        sr_assert_int_equal(ret, SR_ERR_OK);
+        ret = sr_apply_changes(sess, 0);
+        sr_assert_int_equal(ret, SR_ERR_OK);
+    }
+
+    sr_unsubscribe(sub);
+    sr_disconnect(conn);
+    return 0;
+}
+
+static int
+test_pull_push_oper2(int rp, int wp)
+{
+    sr_conn_ctx_t *conn;
+    sr_session_ctx_t *sess;
+    struct lyd_node *data;
+    int ret, i;
+
+    ret = sr_connect(0, &conn);
+    sr_assert_int_equal(ret, SR_ERR_OK);
+
+    ret = sr_session_start(conn, SR_DS_OPERATIONAL, &sess);
+    sr_assert_int_equal(ret, SR_ERR_OK);
+
+    /* wait for the other process */
+    barrier(rp, wp);
+
+    /* keep getting the operational data */
+    for (i = 0; i < 200; ++i) {
+        ret = sr_get_data(sess, "/ietf-interfaces:interfaces/interface", 0, 0, 0, &data);
+        sr_assert_int_equal(ret, SR_ERR_OK);
+        lyd_free_siblings(data);
+    }
+
+    sr_disconnect(conn);
+    return 0;
+}
+
+/* TEST */
+static int
+test_conn_create(int rp, int wp)
+{
+    sr_conn_ctx_t *conn;
+    int ret, i;
+
+    /* wait for the other process */
+    barrier(rp, wp);
+
+    /* keep creating and destroying connections */
+    for (i = 0; i < 10; ++i) {
+        ret = sr_connect(0, &conn);
+        sr_assert_int_equal(ret, SR_ERR_OK);
+
+        sr_disconnect(conn);
+    }
+
+    return 0;
+}
+
 int
 main(void)
 {
@@ -505,10 +623,11 @@ main(void)
         {"rpc sub", test_rpc_sub1, test_rpc_sub2, setup, teardown},
         {"rpc crash", test_rpc_crash1, test_rpc_crash2, setup, teardown},
         {"notif instid", test_notif_instid1, test_notif_instid2, setup, teardown},
+        {"pull push oper data", test_pull_push_oper1, test_pull_push_oper2, setup, teardown},
+        {"conn create", test_conn_create, test_conn_create, setup, teardown},
     };
 
     sr_log_set_cb(test_log_cb);
     run_tests(tests, sizeof tests / sizeof *tests);
     return 0;
 }
-
